#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import argparse

from RouToolPa.Tools.AIndex import AIndex


parser = argparse.ArgumentParser()

parser.add_argument("-f", "--forward_file", action="store", dest="forward_file", required=True,
                    help="File with forward reads")
parser.add_argument("-r", "--reverse_file", action="store", dest="reverse_file", required=True,
                    help="File with reverse reads")
parser.add_argument("-o", "--output_prefix", action="store", dest="output_prefix", required=True,
                    help="Output prefix")
parser.add_argument("-m", "--kmer_length", action="store", dest="kmer_length", type=int, default=23,
                    help="Length of kmers. Default - 23")
parser.add_argument("-j", "--jf_db", action="store", dest="jf_db", required=True,
                    help="Jellyfish kmer database precalculated from same reads ")
parser.add_argument("-t", "--threads", action="store", dest="threads", type=int, default=1,
                    help="Number of threads. Default - 1")
parser.add_argument("-l", "--min_kmer_cov", action="store", dest="min_kmer_coverage", type=int, default=1,
                    help="Minimum coverage of kmers to include. Default - 1(all)")
parser.add_argument("-x", "--max_kmer_cov", action="store", dest="max_kmer_coverage", type=int, default=None,
                    help="Maximum coverage of kmers to include. Default - not set(all)")

args = parser.parse_args()

AIndex.threads = args.threads
AIndex.create_index_from_jf(args.jf_db,
                            args.kmer_length,
                            output_prefix=args.output_prefix,
                            lower_count=args.min_kmer_coverage,
                            upper_count=args.max_kmer_coverage,
                            forward_file=args.forward_file,
                            reverse_file=args.reverse_file,
                            filetype="fastq",
                            create_aindex=True)




#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
import shutil
import numpy as np
from scipy.signal import argrelextrema

import matplotlib
matplotlib.use('Agg')
os.environ['MPLCONFIGDIR'] = '/tmp/'
import matplotlib.pyplot as plt
plt.ioff()
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import MatplotlibRoutines, MathRoutines


class JellyfishRoutines(Tool):
    """
    Several subcommands are not implemented: query, qhisto, qdump, qmerge, cite
    Not all options were implemented for count subcommand

    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "jellyfish", path=path, max_threads=max_threads)

    def scan_for_contamination(self, sequence_file, jf_database, out_file, splited_input_dir="splited_fasta",
                               parsing_mode="parse",splited_full_output_dir="splited_full_output",
                               splited_output_dir="splited_output",
                               splited_sorted_unique_output="splited_sorted_unique_output",
                               retain_intermediate_file=False, ):

        print("Splitting fasta file...")

        self.split_fasta(sequence_file, splited_input_dir, num_of_recs_per_file=1, num_of_files=None,
                         output_prefix="splited_fasta", parsing_mode=parsing_mode)
        options_list = []

        print("Scanning database...")
        self.safe_mkdir(splited_output_dir)
        self.safe_mkdir(splited_full_output_dir)
        self.safe_mkdir(splited_sorted_unique_output)

        for filename in os.listdir(splited_input_dir):
            input_file = "%s/%s" % (splited_input_dir, filename)
            output_file = "%s/%s.kmer.counts" % (splited_output_dir, filename)
            full_output_file = "%s/%s" % (splited_full_output_dir, filename)
            output_sorted_unique_file = "%s/%s.kmer.sorted.unique.counts" % (splited_sorted_unique_output, filename)

            options = "jellyfish query"
            options += " -s %s" % input_file
            options += " %s" % jf_database
            options += " | tee %s | awk '{if ($2 > 0) print $0}' | tee %s | sort | uniq > %s" % (full_output_file,
                                                                                                 output_file,
                                                                                                 output_sorted_unique_file)

            options_list.append(options)

        self.parallel_execute(options_list, cmd="")

        print("Analyzing results...")
        with open(out_file, "w") as out_fd:
            out_fd.write("#record_id\tkmer_number\tcovered_positions\tcovered_positions,%\t"
                         "covered_unique_position\tcovered_unique_positions,%\t"
                         "kmer_mean_coverage\tkmer_median_coverage\tdescription\n")
            for filename in os.listdir(splited_input_dir):
                input_file = "%s/%s" % (splited_input_dir, filename)
                output_file = "%s/%s.kmer.counts" % (splited_output_dir, filename)
                full_output_file = "%s/%s" % (splited_full_output_dir, filename)
                output_sorted_unique_file = "%s/%s.kmer.sorted.unique.counts" % (splited_sorted_unique_output, filename)

                seq_record = self.get_first_record_from_file(input_file, format="fasta")

                covered_positions = 0

                with open(output_file, "r") as a_fd:
                    for line in a_fd:
                        covered_positions += 1

                uniq_covered_positions = 0
                total_kmer_number = 0

                with open(output_sorted_unique_file, "r") as b_fd:
                    for line in b_fd:
                        uniq_covered_positions += 1
                        total_kmer_number += int(line.strip().split()[1])
                kmer_number  = 0
                with open(full_output_file, "r") as a_fd:
                    for line in a_fd:
                        kmer_number += 1
                kmer_coverage = np.loadtxt(full_output_file, usecols=1)
                mean_kmer_coverage = np.mean(kmer_coverage)
                median_kmer_coverage = np.median(kmer_coverage)

                out_fd.write("%s\t%i\t%i\t%.2f\t%i\t%.2f\t%.2f\t%.2f\t%s\n" % (seq_record.id, kmer_number, covered_positions,
                                                                               100*float(covered_positions)/float(kmer_number),
                                                                               uniq_covered_positions,
                                                                               100*float(uniq_covered_positions)/float(kmer_number),
                                                                               mean_kmer_coverage, median_kmer_coverage,
                                                                               seq_record.description))

        if not retain_intermediate_file:
            shutil.rmtree(splited_input_dir)
            shutil.rmtree(splited_output_dir)
            shutil.rmtree(splited_sorted_unique_output)

    def draw_kmer_distribution(self, histo_file_list, kmer_length, output_prefix, label_list=None, output_formats=["svg", "png"],
                               logbase=10, non_log_low_limit=5, non_log_high_limit=100, order=3, mode="wrap",
                               check_peaks_coef=10, draw_separated_pictures=False,
                               use_second_peak_for_genome_size_estimation=False, dont_show_genome_size_on_plot=False):

        data_list = []
        parameters_list = []

        stat_fd = open("%s.histo.stats" % output_prefix, "w")
        max_selected_counts = 0
        max_counts = 0

        default_labels = map(lambda ind: "S%i" % ind, range(1, len(histo_file_list) + 1))

        histo_file_listtt = [histo_file_list] if isinstance(histo_file_list, str) else histo_file_list
        label_listtt = [label_list] if isinstance(label_list, str) else label_list

        for histo_file, label in zip(histo_file_listtt, label_listtt if label_listtt else default_labels):
            bins, counts = np.loadtxt(histo_file, unpack=True)
            data_list.append((bins, counts))

            maximums_to_show, minimums_to_show, \
                unique_peak_borders, number_of_distinct_kmers, \
                number_of_distinct_kmers_with_errors,\
                total_number_of_kmers, total_number_of_kmers_with_errors, \
                estimated_genome_size = self.extract_parameters_from_histo(counts, bins,
                                                                           output_prefix,
                                                                           order=order,
                                                                           mode=mode,
                                                                           check_peaks_coef=check_peaks_coef,
                                                                           use_second_peak_for_genome_size_estimation=use_second_peak_for_genome_size_estimation)

            parameters_list.append((maximums_to_show,
                                    minimums_to_show,
                                    unique_peak_borders,
                                    number_of_distinct_kmers,
                                    number_of_distinct_kmers_with_errors,
                                    total_number_of_kmers,
                                    total_number_of_kmers_with_errors,
                                    estimated_genome_size))

            unique_peak_width = unique_peak_borders[1] - unique_peak_borders[0] + 1
            #print unique_peak_borders
            #print unique_peak_width
            #print "Maximums to show"
            #print maximums_to_show
            #print "Minimums to show"
            #print minimums_to_show

            unique_peak_borders_mean_multiplicity = MathRoutines.mean_from_bins(bins[unique_peak_borders[0]: unique_peak_borders[1]+1],
                                                                                counts[unique_peak_borders[0]: unique_peak_borders[1]+1])
            std_1 = MathRoutines.std_from_bins(bins[unique_peak_borders[0]: unique_peak_borders[1]+1],
                                               counts[unique_peak_borders[0]: unique_peak_borders[1]+1],
                                               mean=unique_peak_borders_mean_multiplicity)
            var_1 = std_1 / unique_peak_borders_mean_multiplicity

            fraction_of_distinct_kmers_with_errors = float(number_of_distinct_kmers_with_errors)/float(number_of_distinct_kmers)
            fraction_of_kmers_with_errors = float(total_number_of_kmers_with_errors)/float(total_number_of_kmers)

            general_stats = "Sample\t%s\n" % label
            general_stats += "Number of distinct kmers\t%i\n" % number_of_distinct_kmers
            general_stats += "Number of distinct kmers with errors\t%i\n" % number_of_distinct_kmers_with_errors
            general_stats += "Fraction of distinct kmers with errors\t%.3f\n" % np.around(fraction_of_distinct_kmers_with_errors, decimals=3)
            general_stats += "Total number of kmers\t%i\n" % total_number_of_kmers
            general_stats += "Total number of kmers with errors\t%i\n" % total_number_of_kmers_with_errors
            general_stats += "Fraction of kmers with errors\t%.3f\n" % np.around(fraction_of_kmers_with_errors, decimals=3)
            general_stats += "Kmer multiplicity at first minimum\t%s\n" % (str(minimums_to_show[0][0]) if minimums_to_show else "None")
            general_stats += "Kmer multiplicity at first maximum\t%s\n" % (str(maximums_to_show[0][0]) if maximums_to_show else "None")
            general_stats += "Width of first peak\t%i\n" % unique_peak_width
            general_stats += "Mean kmer multiplicity in first peak\t%.2f\n" % np.around(unique_peak_borders_mean_multiplicity, decimals=2)
            general_stats += "Standard deviation of kmer multiplicity in first peak\t%.2f\n" % np.around(std_1, decimals=2)
            general_stats += "Variance coefficient of kmer multiplicity in first peak\t%.2f\n" % np.around(var_1,
                                                                                                           decimals=2)
            general_stats += "Estimated genome size, bp\t%s\n" % ("NA" if estimated_genome_size is None else str(estimated_genome_size))
            #with open("%s.histo.stats" % output_prefix, "w") as stat_fd:

            stat_fd.write(general_stats)
            print(general_stats)
            print("\n")

            max_bin = max(bins)

            selected_counts = counts[non_log_low_limit-1:non_log_high_limit]
            selected_bins = bins[non_log_low_limit-1:non_log_high_limit]

            current_max_selected_counts = max(selected_counts)
            current_max_counts = max(counts)

            max_selected_counts = current_max_selected_counts if current_max_selected_counts > max_selected_counts else max_selected_counts
            max_counts = current_max_counts if current_max_counts > max_counts else max_counts

            if draw_separated_pictures:
                figure = plt.figure(1, figsize=(5, 5), dpi=300)
                subplot = plt.subplot(1, 1, 1)
                plt.suptitle("Distribution of %i-mers" % kmer_length, fontweight='bold')
                plt.plot(bins, counts)
                plt.xlim(xmin=1, xmax=max_bin)
                plt.ylim(ymin=1, ymax=max_counts)
                plt.xlabel("Multiplicity")
                plt.ylabel("Number of distinct %s-mers" % kmer_length)
                subplot.set_yscale('log', basey=logbase)
                subplot.set_xscale('log', basex=logbase)

                figure = plt.figure(2, figsize=(5, 5), dpi=300)
                subplot = plt.subplot(1, 1, 1)
                plt.suptitle("Distribution of %s-mers" % kmer_length, fontweight='bold')
                plt.plot(selected_bins, selected_counts)

                plt.xlabel("Multiplicity")
                plt.ylabel("Number of distinct %s-mers" % kmer_length)
                plt.xlim(xmin=non_log_low_limit, xmax=non_log_high_limit)
                plt.ylim(ymin=0, ymax=max_selected_counts)

            if estimated_genome_size is None:
                legend = "NA"
            else:
                size_in_terabases = float(estimated_genome_size) / float(10 ** 12)
                size_in_gigabases = float(estimated_genome_size) / float(10 ** 9)
                size_in_megabases = float(estimated_genome_size) / float(10 ** 6)
                size_in_kilobases = float(estimated_genome_size) / float(10 ** 3)

                if size_in_terabases > 1:
                    legend = "%s: %.2f Tbp" % (label, size_in_terabases)
                elif size_in_gigabases > 1:
                    legend = "%s: %.2f Gbp" % (label, size_in_gigabases)
                elif size_in_megabases > 1:
                    legend = "%s: %.2f Mbp" % (label, size_in_megabases)
                else:
                    legend = "%s: %.2f Kbp" % (label, size_in_kilobases)

            for index in range(3, 5):
                figure = plt.figure(index, figsize=(5, 10))
                subplot_list = []
                for i, b, c in zip([1, 2], [bins, selected_bins], [counts, selected_counts]):
                    subplot_list.append(plt.subplot(2, 1, i))
                    plt.suptitle("Distribution of %s-mers" % kmer_length, fontweight='bold', fontsize=13)
                    plt.plot(b, c, label=legend) if (i == 1) and (not dont_show_genome_size_on_plot) else plt.plot(b, c)

                    #plt.legend((legend, ), loc="upper right")
                    #plt.legend(loc="upper right")
                    if i == 1:
                        plt.legend(loc="best")

                    if index == 4:
                        for minimum in minimums_to_show:
                            plt.plot([minimum[0], minimum[0]], [0, minimum[1]], 'r--', lw=1)
                        for maximum in maximums_to_show:
                            plt.plot([maximum[0], maximum[0]], [0, maximum[1]], 'g--', lw=1)

                    plt.ylabel("Number of distinct %s-mers" % kmer_length, fontsize=13)

                    if i == 1:
                        subplot_list[0].set_yscale('log', basey=logbase)
                        subplot_list[0].set_xscale('log', basex=logbase)
                        plt.xlim(xmin=1, xmax=max_bin)
                        plt.ylim(ymin=1, ymax=max_counts)
                    elif i == 2:
                        plt.ylim(ymin=0, ymax=max_selected_counts)
                        plt.xlim(xmin=non_log_low_limit, xmax=non_log_high_limit)
                        plt.xlabel("Multiplicity", fontsize=15)

                MatplotlibRoutines.zoom_effect(subplot_list[0], subplot_list[1], non_log_low_limit, non_log_high_limit)
                plt.subplots_adjust(hspace=0.12, wspace=0.05, top=0.95, bottom=0.05, left=0.16, right=0.95)

        if draw_separated_pictures:
            for index, name_template in ((1, "%s.logscale.%s"), (2, "%s.no_logscale.%s")):
                plt.figure(index)
                for extension in output_formats:
                    plt.savefig(name_template % (output_prefix, extension))
                plt.close()

        for index in range(3, 5):
            plt.figure(index)
            for extension in output_formats:
                plt.savefig("%s.%s" % ("%s.peaks_and_gaps" % output_prefix if index == 4 else output_prefix, extension))

            plt.close()

    @staticmethod
    def find_peak_indexes_from_histo(counts, order=3, mode="wrap"):
        """
        order:
            How many points on each side to use for the comparison to consider comparator(n, n+x) to be True.
        mode:
            How the edges of the vector are treated. 'wrap' (wrap around) or 'clip' (treat overflow as the same
            as the last
        """
        local_maximums_idx = argrelextrema(counts, np.greater, order=order, mode=mode)[0]
        local_minimums_idx = argrelextrema(counts, np.less, order=order, mode=mode)[0]

        return local_minimums_idx, local_maximums_idx

    @staticmethod
    def extract_parameters_from_histo(counts, bins, output_prefix, order=3, mode="wrap", check_peaks_coef=10,
                                      use_second_peak_for_genome_size_estimation=False):
        """
        check_peaks_coef:
            histogram is checked for presence of additional peaks in range [first_unique_peak, check_peaks_coef*first_unique_peak]
        """

        local_maximums_idx = argrelextrema(counts, np.greater, order=order, mode=mode)[0]
        local_minimums_idx = argrelextrema(counts, np.less, order=order, mode=mode)[0]

        with open("%s.local_maximums" % output_prefix, "w") as out_fd:
            out_fd.write("#multiplicity\tnumber_of_kmers\n")
            for idx in local_maximums_idx:
                out_fd.write("%i\t%i\n" % (bins[idx], counts[idx]))

        with open("%s.local_minimums" % output_prefix, "w") as out_fd:
            out_fd.write("#multiplicity\tnumber_of_kmers\n")
            for idx in local_minimums_idx:
                out_fd.write("%i\t%i\n" % (bins[idx], counts[idx]))

        first_unique_peak_idx_idx = 0 if local_maximums_idx[0] != 0 else 1
        #second_unique_peak_idx_idx = 1 if local_maximums_idx[1] != 0 else 2
        first_unique_peak_coverage = bins[local_maximums_idx[first_unique_peak_idx_idx]]
        second_unique_peak_coverage = bins[local_maximums_idx[first_unique_peak_idx_idx+1]]

        max_checked_coverage = check_peaks_coef * first_unique_peak_coverage
        peaks_in_checked_area_idx = [local_maximums_idx[first_unique_peak_idx_idx]]
        minimums_in_checked_area_idx = []

        for i in range(first_unique_peak_idx_idx+1, len(local_maximums_idx)):
            if bins[local_maximums_idx[i]] <= max_checked_coverage:
                peaks_in_checked_area_idx.append(local_maximums_idx[i])
            else:
                break

        for minimum_index in local_minimums_idx:
            if bins[minimum_index] <= max_checked_coverage:
                minimums_in_checked_area_idx.append(minimum_index)

        if len(peaks_in_checked_area_idx) > 1:
            print ("WARNING! Additional k-mer peaks were detected with multiplicity (%i, %i]" % (first_unique_peak_coverage,
                                                                                                 max_checked_coverage))

        nearest_value_to_first_min_idx = MathRoutines.find_nearest_scalar(counts[local_maximums_idx[first_unique_peak_idx_idx]:],
                                                                          counts[local_minimums_idx[0]]) + local_maximums_idx[first_unique_peak_idx_idx]

        number_of_distinct_kmers = sum(counts)
        number_of_distinct_kmers_with_errors = sum(counts[0:local_minimums_idx[0]])
        total_number_of_kmers = sum(np.multiply(counts, bins))
        total_number_of_kmers_with_errors = sum(np.multiply(counts[0:local_minimums_idx[0]],
                                                                     bins[0:local_minimums_idx[0]]))

        maximums_to_show = [(bins[i], counts[i]) for i in peaks_in_checked_area_idx]
        minimums_to_show = [(bins[i], counts[i]) for i in minimums_in_checked_area_idx]

        estimated_genome_size = 0
        """
        print local_minimums_idx
        print local_maximums_idx
        print minimums_to_show
        """
        if minimums_to_show:
            for i in range(int(minimums_to_show[0][0]), len(counts)):
                estimated_genome_size += counts[i] * bins[i]
        else:
            print("WARNING: Minimum between error peak and unique peak was not found. "
                  "Estimation of genome size is impossible ")
            estimated_genome_size = None

        genome_coverage_peak = second_unique_peak_coverage if use_second_peak_for_genome_size_estimation else first_unique_peak_coverage

        # print genome_coverage_peak

        estimated_genome_size = estimated_genome_size/genome_coverage_peak if estimated_genome_size else None

        return maximums_to_show, \
               minimums_to_show, \
               (local_minimums_idx[0], nearest_value_to_first_min_idx), \
               number_of_distinct_kmers, number_of_distinct_kmers_with_errors, \
               total_number_of_kmers, total_number_of_kmers_with_errors, estimated_genome_size

if __name__ == "__main__":
    pass



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
#import os
import argparse

from KRATER.Routines import JellyfishRoutines


parser = argparse.ArgumentParser()

parser.add_argument("-i", "--input", action="store", dest="input", required=True, type=lambda s: s.split(","),
                    help="Comma-separated list of input files with data for histogram")
parser.add_argument("-a", "--labels", action="store", dest="labels", type=lambda s: s.split(","),
                    help="Comma-separated list of labels for input files")
parser.add_argument("-o", "--output_prefix", action="store", dest="output_prefix", required=True,
                    help="Output prefix")
parser.add_argument("-e", "--output_formats", action="store", dest="output_formats", type=lambda s: s.split(","),
                    default=["png"],
                    help="Comma-separated list of formats (supported by matlotlib) "
                         "of output figure.Default: png")
parser.add_argument("-l", "--logbase", action="store", dest="logbase", type=int, default=10,
                    help="Base of logarithm. Default - 10")
parser.add_argument("-w", "--low_limit", action="store", dest="low_limit", type=int, default=5,
                    help="Low limit of histogram without logscale")
parser.add_argument("-g", "-high_limit", action="store", dest="high_limit", type=int, default=100,
                    help="High limit of histogram without logscale")
parser.add_argument("-m", "--kmer_length", action="store", dest="kmer_length", type=int, required=True,
                    help="Length of kmer used to construct histo file.")
parser.add_argument("-d", "--draw_separated_pictures", action="store_true", dest="draw_separated_pictures",
                    default=False,
                    help="Draw additional separated pictures for double logarithmic and linear scales."
                         "Default - False")
parser.add_argument("-u", "--point_number", action="store", dest="point_number", default=3, type=int,
                    help="Number of values on both sides from each bin in histogram to count for detection "
                         "of maximums and minimums. Rise if you histogram is noisy. Default: 3")
parser.add_argument("-z", "--use_second_peak", action="store_true", dest="use_second_peak", default=False,
                    help="Use second peak from histogram for genome size estimation")
parser.add_argument("--dont_show_genome_size_on_plot", action="store_true", dest="dont_show_genome_size_on_plot", default=False,
                    help="Dont show genome size on plot. Default: False")
#parser.add_argument("-d", "--draw_peaks_and_gaps", action="store_true", dest="draw_peaks_and_gaps",
#                    help="Draw peaks and gaps")

args = parser.parse_args()

JellyfishRoutines.draw_kmer_distribution(args.input,
                                         args.kmer_length,
                                         args.output_prefix,
                                         output_formats=args.output_formats,
                                         logbase=args.logbase,
                                         non_log_low_limit=args.low_limit,
                                         label_list=args.labels,
                                         non_log_high_limit=args.high_limit,
                                         order=args.point_number,
                                         draw_separated_pictures=args.draw_separated_pictures,
                                         use_second_peak_for_genome_size_estimation=args.use_second_peak,
                                         dont_show_genome_size_on_plot=args.dont_show_genome_size_on_plot) #, draw_peaks_and_gaps=args.draw_peaks_and_gaps)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import argparse
from collections import OrderedDict
from RouToolPa.Parsers.KrATER import KrATERReportCollection

parser = argparse.ArgumentParser()

parser.add_argument("-i", "--input", action="store", dest="input", required=True,
                    type=lambda s: s.split(","),
                    help="Comma-separated list of files with KrATER reports")
parser.add_argument("-s", "--samples", action="store", dest="samples",
                    type=lambda s: s.split(","),
                    help="Comma-separated list of sample ids. Must have same length as list of files."
                         "If not set filenames will be treated as sample ids")
parser.add_argument("-o", "--output", action="store", dest="output", required=True,
                    help="File to write general statistics")

args = parser.parse_args()

filename_dict = OrderedDict()
if args.samples:
    for i in range(0, len(args.input)):
        filename_dict[args.samples[i]] = args.input[i]
else:
    for filename in args.input:
        filename_dict[filename] = filename

report_collection = KrATERReportCollection(file_dict=filename_dict)

report_collection.write_general_stats(args.output)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
#import os
import argparse

from KRATER.Routines import AIndexRoutines


parser = argparse.ArgumentParser()

parser.add_argument("-s", "--sequences", action="store", dest="seqs", required=True,
                    help="Input fasta file with representative contaminating sequences, for example 18S rRNA")
parser.add_argument("-i", "--index_prefix", action="store", dest="index_prefix", required=True,
                    help="Index prefix")
parser.add_argument("-k", "--kmer_length", action="store", dest="kmer_len", required=True,
                    type=int,
                    help="Kmer length used for index creation")
parser.add_argument("-o", "--output", action="store", dest="output", required=True,
                    help="Output file")

parser.add_argument("-t", "--threads", action="store", dest="threads", type=int, default=1,
                    help="Number of threads to use. Default: 1")

args = parser.parse_args()

AIndexRoutines.threads = args.threads
AIndexRoutines.scan_for_contamination(args.seqs,
                                      args.index_prefix,
                                      args.kmer_len,
                                      args.output,
                                      aindex_prefix=None,
                                      reads_file=None,
                                      parsing_mode="parse",
                                      external_process_pool=None,
                                      threads=None)



__author__ = 'Sergei F. Kliver'

from KRATER.Routines.AIndexRoutines import AIndexRoutines
from KRATER.Routines.JellyfishRoutines import JellyfishRoutines

AIndexRoutines = AIndexRoutines()
JellyfishRoutines = JellyfishRoutines()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
import argparse

from Bio import SeqIO

from RouToolPa.Routines.Sequence import rev_com_generator
from RouToolPa.Tools.Kmers import Jellyfish

from KRATER.Routines import JellyfishRoutines

parser = argparse.ArgumentParser()

parser.add_argument("-i", "--input_file", action="store", dest="input", type=lambda s: s.split(","), required=True,
                    help="Comma-separated list of fasta or fastq files or directories containing them.")
parser.add_argument("-o", "--output_prefix", action="store", dest="output_prefix", required=True,
                    help="Output prefix")
parser.add_argument("-e", "--output_formats", action="store", dest="output_formats", type=lambda s: s.split(","),
                    default=["png"],
                    help="Comma-separated list of formats (supported by matlotlib) "
                         "of output figure.Default: png")
parser.add_argument("-l", "--logbase", action="store", dest="logbase", type=int, default=10,
                    help="Base of logarithm. Default - 10")
parser.add_argument("-m", "--kmer_length", action="store", dest="kmer_length", type=int, default=23,
                    help="Length of kmers. Default - 23")
parser.add_argument("-s", "--hash_size", action="store", dest="hash_size", type=str, default="1000000",
                    help="Size of hash. Estimation of hash size: for short reads S=(G + k*n)/0.8, "
                    "G - genome size, k - kmer length, n - number of reads, for assembled sequences "
                    "S=Sum(L). Default - 1000000")
parser.add_argument("-t", "--threads", action="store", dest="threads", type=int, default=1,
                    help="Number of threads. Default - 1")
parser.add_argument("-b", "--count_both_strands", action="store_true", dest="count_both_strands",
                    help="Count kmers in both strands. NOTICE: only mer or its reverse-complement, whichever "
                         "comes first lexicographically, is stored and the count value is the number of "
                         "occurrences of both. So this option is not suitable for generating sets of forward "
                         "and reverse-complement kmers. For this case use -r/--add_reverse_complement option. "
                         "Not compatible with -r/--add_reverse_complement option.")
parser.add_argument("-r", "--add_reverse_complement", action="store_true", dest="add_rev_com",
                    help="Add reverse-complement sequences before counting kmers. "
                         "Works only for fasta sequences. "
                         "Not compatible with -b/--count_both_strands option")
parser.add_argument("-j", "--jellyfish_path", action="store", dest="jellyfish_path",
                    help="Path to jellyfish")
parser.add_argument("-w", "--low_limit", action="store", dest="low_limit", type=int, default=5,
                    help="Low limit of histogram without logscale. Default - 5")
parser.add_argument("-g", "--high_limit", action="store", dest="high_limit", type=int, default=100,
                    help="High limit of histogram without logscale. Default - 100")
parser.add_argument("-d", "--draw_separated_pictures", action="store_true", dest="draw_separated_pictures",
                    default=False,
                    help="Draw additional separated pictures for double logarithmic and linear scales."
                         "Default - False")
parser.add_argument("-u", "--point_number", action="store", dest="point_number", default=3, type=int,
                    help="Number of values on both sides from each bin in histogram to count for detection "
                         "of maximums and minimums. Rise if you histogram is noisy. Default: 3")
parser.add_argument("-z", "--use_second_peak", action="store_true", dest="use_second_peak", default=False,
                    help="Use second peak from histogram for genome size estimation")
parser.add_argument("--dont_show_genome_size_on_plot", action="store_true", dest="dont_show_genome_size_on_plot", default=False,
                    help="Dont show genome size on plot. Default: False")
parser.add_argument("--turn_on_timelog", action="store_true", dest="turn_on_timelog", default=False,
                    help="Turn on timelog. Default: False")
parser.add_argument("--generators", action="store", dest="generators",
                    help="File with commands for fastq generators. Default: None")
#parser.add_argument("-d", "--draw_peaks_and_gaps", action="store_true", dest="draw_peaks_and_gaps",
#                    help="Draw peaks and gaps")

args = parser.parse_args()

args.input = Jellyfish.make_list_of_path_to_files(args.input)
if args.count_both_strands and args.add_rev_com:
    raise ValueError("Options -b/--count_both_strands and -r/--add_reverse_complement are not compatible")

if args.add_rev_com:
    file_with_rev_com = args.output_prefix + "_with_rev_com.fasta"
    record_dict = SeqIO.index_db("temp_index.idx", args.input, format="fasta")
    SeqIO.write(rev_com_generator(record_dict, yield_original_record=True), file_with_rev_com, "fasta")
    args.output_prefix += "_with_rev_com"
    os.remove("temp_index.idx")

output_prefix = "%s.k%i" % (args.output_prefix, args.kmer_length)
base_file = "%s.jf" % output_prefix
kmer_table_file = "%s.counts" % output_prefix
kmer_file = "%s.kmer" % output_prefix

histo_file = "%sr.histo" % output_prefix
picture_prefix = "%s.histogram" % output_prefix

Jellyfish.threads = args.threads
Jellyfish.timelog = "%s.jellyfish.time.log" % output_prefix if args.turn_on_timelog else None
Jellyfish.path = args.jellyfish_path if args.jellyfish_path else ""
Jellyfish.count(args.input if not args.add_rev_com else file_with_rev_com, base_file,
                kmer_length=args.kmer_length, hash_size=args.hash_size,
                count_both_strands=args.count_both_strands,
                generators=args.generators)
Jellyfish.histo(base_file, histo_file, upper_count=100000000)
JellyfishRoutines.draw_kmer_distribution(histo_file, args.kmer_length, picture_prefix,
                                         output_formats=args.output_formats,
                                         logbase=args.logbase, non_log_low_limit=args.low_limit,
                                         non_log_high_limit=args.high_limit, order=args.point_number,
                                         use_second_peak_for_genome_size_estimation=args.use_second_peak,
                                         draw_separated_pictures=args.draw_separated_pictures,
                                         dont_show_genome_size_on_plot=args.dont_show_genome_size_on_plot) #, draw_peaks_and_gaps=args.draw_peaks_and_gaps)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
#import os
import argparse

from KRATER.Routines import JellyfishRoutines


parser = argparse.ArgumentParser()

parser.add_argument("-i", "--input", action="store", dest="input", required=True,
                    help="Input fasta file with representative contaminating sequences, for example 18S rRNA")
parser.add_argument("-j", "--jellyfish_db", action="store", dest="jf_db", required=True,
                    help="Jellyfish database")
parser.add_argument("-o", "--output", action="store", dest="output", required=True,
                    help="Output file")
parser.add_argument("-r", "--retain_intermediate_files", action="store_true", dest="retain_intermediate_files",
                    default=False, help="Store intermediate files")
parser.add_argument("-t", "--threads", action="store", dest="threads", type=int, default=1,
                    help="Number of threads to use. Default: 1")

args = parser.parse_args()

JellyfishRoutines.threads = args.threads
JellyfishRoutines.scan_for_contamination(args.input, args.jf_db, args.output, splited_input_dir="splited_fasta",
                                         parsing_mode="parse", splited_output_dir="splited_output",
                                         splited_sorted_unique_output="splited_sorted_unique_output",
                                         retain_intermediate_file=args.retain_intermediate_files)



__author__ = 'Sergei F. Kliver'
import os
from setuptools import setup, find_packages


def get_list_of_files(list_of_dirs_and_files, expression=None, recursive=True):
    file_list = []

    for entry in [list_of_dirs_and_files] if isinstance(list_of_dirs_and_files, str) else list_of_dirs_and_files:

        if os.path.isdir(entry):

            files_in_dir = ["%s%s" % (entry if entry[-1] == "/" else (entry+"/"), filename)
                            for filename in sorted(filter(expression, os.listdir(entry))
                                                   if expression else os.listdir(entry))]
            if recursive:
                for filename in files_in_dir:
                    if os.path.isdir(filename):
                        file_list += get_list_of_files([filename], expression=expression, recursive=True)
                    else:
                        file_list.append(filename)
            else:
                file_list += files_in_dir
        elif os.path.exists(entry):
            file_list.append(os.path.abspath(entry))
        else:
            print("%s does not exist" % entry)
    return file_list


setup(name='KrATER',
      version='2.0b',
      packages=find_packages(),
      author='Sergei F. Kliver',
      url='https://github.com/mahajrod/KRATER',
      author_email='mahajrod@gmail.com',
      scripts=get_list_of_files("scripts/"),
      install_requires=['scipy', 'numpy', 'matplotlib', 'biopython', 'RouToolPa'],
      long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read())
      #scripts=['draw_kmer_distribution_from_fastq.py',
      #         'draw_kmer_distribution_from_histo.py',
      #         'draw_kmer_distribution_from_jellyfish_database.py'])



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
import shutil
import numpy as np
import multiprocessing as mp

import matplotlib
matplotlib.use('Agg')
os.environ['MPLCONFIGDIR'] = '/tmp/'
import matplotlib.pyplot as plt
plt.ioff()

try:
    from aindex import *
except:
    print("WARNING!!! aindex package was not found! Related code will not work")

from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Parsers.Sequence import CollectionSequence


class AIndexRoutines(Tool):

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "aindex", path=path, max_threads=max_threads)

    def scan_for_contamination(self,
                               sequence_file,
                               index_prefix,
                               kmer_length,
                               output_file,
                               aindex_prefix=None,
                               reads_file=None,
                               parsing_mode="parse",
                               external_process_pool=None,
                               threads=None):

        index_settings = {
                          "index_prefix": index_prefix,
                          "aindex_prefix": aindex_prefix,
                          "reads_file": reads_file,
                          }

        print("Parsing fasta file...")

        sequence_collection = CollectionSequence(in_file=sequence_file, parsing_mode=parsing_mode)
        sequence_collection.get_stats_and_features(count_gaps=False, sort=False, min_gap_length=1)

        index = load_aindex(index_settings,
                            skip_reads=False if reads_file else True,
                            skip_aindex=False if reads_file else True)

        results = []
        counter = 0
        for seq_id in sequence_collection.scaffolds:
            #print sequence_collection.seq_lengths.at[seq_id, "length"]
            results.append((seq_id, [index[sequence_collection.records[seq_id][i:i + kmer_length]] \
                           for i in xrange(sequence_collection.seq_lengths.at[seq_id, "length"] - kmer_length + 1)]))
            counter += 1
            if counter % 1000 == 0:
                print("%i sequences were handled..." % counter)
        """
        def get_kmer_coverage(seq_tuple):
            return (seq_tuple[0], [index[seq_tuple[1][i:i + kmer_length]] \
                             for i in xrange(len(seq_tuple[1]) - kmer_length + 1)])

        process_pool = external_process_pool if external_process_pool else mp.Pool(threads if threads else self.threads)

        results = process_pool.map(get_kmer_coverage, sequence_collection.scaffolds, chunksize=1)
        """
        print("Analyzing results...")
        with open(output_file, "w") as out_fd:
            out_fd.write("#record_id\tkmer_number\tcovered_positions\tcovered_positions,%\t"
                         "kmer_mean_coverage\tkmer_median_coverage\tdescription\n")
            for seq_id, kmer_coverage_list in results:
                kmer_number = sequence_collection.seq_lengths.at[seq_id, "length"]-kmer_length+1
                covered_positions = kmer_number - kmer_coverage_list.count(0)

                mean_kmer_coverage = np.mean(kmer_coverage_list)
                median_kmer_coverage = np.median(kmer_coverage_list)

                out_fd.write("%s\t%i\t%i\t%.2f\t%.2f\t%.2f\t%s\n" % (seq_id,
                                                                     kmer_number,
                                                                     covered_positions,
                                                                     100*float(covered_positions)/float(kmer_number),
                                                                     mean_kmer_coverage,
                                                                     median_kmer_coverage,
                                                                     sequence_collection.description[seq_id]))

